@@ -11,6 +11,7 @@ import (
 	"math/rand"
 	"net"
 	"net/http"
+	"net/url"
 	"os"
 	"os/signal"
 	"slices"
@@ -20,6 +21,7 @@ import (
 	"syscall"
 	"time"
 
+	"github.com/glasskube/glasskube/internal/dependency/graph"
 	"github.com/glasskube/glasskube/internal/telemetry/annotations"
 
 	"github.com/glasskube/glasskube/internal/web/components/toast"
@@ -232,6 +234,7 @@ func (s *server) Start(ctx context.Context) error {
 	router.Handle("/datalists/{valueName}/keys", s.requireReady(s.keysDatalist))
 	// settings
 	router.Handle("/settings", s.requireReady(s.settingsPage))
+	router.Handle("/settings/repository/{repoName}", s.requireReady(s.repositoryConfig))
 	router.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
 		http.Redirect(w, r, "/clusterpackages", http.StatusFound)
 	})
@@ -418,7 +421,7 @@ func (s *server) uninstall(w http.ResponseWriter, r *http.Request) {
 				s.sendToast(w, toast.WithErr(fmt.Errorf("failed to fetch clusterpackage %v: %w", pkgName, err)))
 				return
 			}
-			if err := uninstaller.Uninstall(ctx, &pkg); err != nil {
+			if err := uninstaller.Uninstall(ctx, &pkg, false); err != nil {
 				s.sendToast(w, toast.WithErr(fmt.Errorf("failed to uninstall clusterpackage %v: %w", pkgName, err)))
 				return
 			}
@@ -428,20 +431,19 @@ func (s *server) uninstall(w http.ResponseWriter, r *http.Request) {
 				s.sendToast(w, toast.WithErr(fmt.Errorf("failed to fetch package %v/%v: %w", namespace, name, err)))
 				return
 			}
-			if err := uninstaller.Uninstall(ctx, &pkg); err != nil {
+			if err := uninstaller.Uninstall(ctx, &pkg, false); err != nil {
 				s.sendToast(w, toast.WithErr(fmt.Errorf("failed to uninstall package %v/%v: %w", namespace, name, err)))
 				return
 			}
 		}
 	} else {
 		if pkgName != "" {
-			var pruned []string
+			var pruned []graph.PackageRef
 			var err error
-			// dependency checks are only necessary for clusterpackages, as there are no dependencies on namespaced packages
 			if g, err1 := s.dependencyMgr.NewGraph(r.Context()); err1 != nil {
 				err = fmt.Errorf("error validating uninstall: %w", err1)
 			} else {
-				g.Delete(pkgName)
+				g.Delete(pkgName, "")
 				pruned = g.Prune()
 				if err1 := g.Validate(); err1 != nil {
 					err = fmt.Errorf("%v cannot be uninstalled: %w", pkgName, err1)
@@ -456,9 +458,23 @@ func (s *server) uninstall(w http.ResponseWriter, r *http.Request) {
 			})
 			util.CheckTmplError(err, "pkgUninstallModalTmpl")
 		} else {
-			err := s.templates.pkgUninstallModalTmpl.Execute(w, map[string]any{
+			var pruned []graph.PackageRef
+			var err error
+			// TODO: refactor this duplicate code segment
+			if g, err1 := s.dependencyMgr.NewGraph(r.Context()); err1 != nil {
+				err = fmt.Errorf("error validating uninstall: %w", err1)
+			} else {
+				g.Delete(name, namespace)
+				pruned = g.Prune()
+				if err1 := g.Validate(); err1 != nil {
+					err = fmt.Errorf("%v cannot be uninstalled: %w", pkgName, err1)
+				}
+			}
+			err = s.templates.pkgUninstallModalTmpl.Execute(w, map[string]any{
 				"Namespace":   namespace,
 				"Name":        name,
+				"Pruned":      pruned,
+				"Err":         err,
 				"PackageHref": util.GetNamespacedPkgHref(manifestName, namespace, name),
 				"GitopsMode":  s.isGitopsModeEnabled(),
 			})
@@ -899,10 +915,25 @@ func (s *server) handleAdvancedConfig(ctx context.Context, d *packageDetailPageC
 			d.selectedVersion = latestVersion
 		}
 
-		res, err := s.dependencyMgr.Validate(r.Context(), d.manifest, d.selectedVersion)
-		if err != nil {
+		var validatinoResult *dependency.ValidationResult
+		var validationErr error
+		if d.pkg.IsNil() {
+			if d.manifest.Scope.IsCluster() {
+				validatinoResult, validationErr =
+					s.dependencyMgr.Validate(r.Context(), d.manifestName, "", d.manifest, d.selectedVersion)
+			} else {
+				// In this case we don't know the actual namespace, but we can assume the default
+				// TODO: make name and namespace depend on user input
+				validatinoResult, validationErr =
+					s.dependencyMgr.Validate(r.Context(), d.manifestName, d.manifest.DefaultNamespace, d.manifest, d.selectedVersion)
+			}
+		} else {
+			validatinoResult, validationErr =
+				s.dependencyMgr.Validate(r.Context(), d.pkg.GetName(), d.pkg.GetNamespace(), d.manifest, d.selectedVersion)
+		}
+		if validationErr != nil {
 			s.sendToast(w,
-				toast.WithErr(fmt.Errorf("failed to validate dependencies of %v in version %v: %w", d.manifestName, d.selectedVersion, err)))
+				toast.WithErr(fmt.Errorf("failed to validate dependencies of %v in version %v: %w", d.manifestName, d.selectedVersion, validationErr)))
 			return
 		}
 
@@ -910,8 +941,8 @@ func (s *server) handleAdvancedConfig(ctx context.Context, d *packageDetailPageC
 			"Status":           client.GetStatusOrPending(d.pkg),
 			"Manifest":         d.manifest,
 			"LatestVersion":    latestVersion,
-			"ValidationResult": res,
-			"ShowConflicts":    res.Status == dependency.ValidationResultStatusConflict,
+			"ValidationResult": validatinoResult,
+			"ShowConflicts":    validatinoResult.Status == dependency.ValidationResultStatusConflict,
 			"SelectedVersion":  d.selectedVersion,
 			"PackageIndex":     &idx,
 			"Repositories":     repos,
@@ -1044,6 +1075,85 @@ func (s *server) settingsPage(w http.ResponseWriter, r *http.Request) {
 	util.CheckTmplError(tmplErr, "settings")
 }
 
+func (s *server) repositoryConfig(w http.ResponseWriter, r *http.Request) {
+	switch r.Method {
+	case http.MethodGet:
+		s.getHandleRepositoryConfig(w, r)
+	case http.MethodPost:
+		s.getUpdateRepositoryConfig(w, r)
+	}
+
+}
+
+func (s *server) getHandleRepositoryConfig(w http.ResponseWriter, r *http.Request) {
+	repoName := mux.Vars(r)["repoName"]
+	var repo v1alpha1.PackageRepository
+	if err := s.pkgClient.PackageRepositories().Get(r.Context(), repoName, &repo); err != nil {
+		// error handling
+		s.sendToast(w, toast.WithErr(fmt.Errorf("failed to fetch repositories: %w", err)))
+		return
+	}
+	tmplErr := s.templates.repositoryPageTmpl.Execute(w, s.enrichPage(r, map[string]any{
+		"Repository": repo,
+	}, nil))
+	util.CheckTmplError(tmplErr, "repository")
+
+}
+
+func (s *server) getUpdateRepositoryConfig(w http.ResponseWriter, r *http.Request) {
+	repoName := mux.Vars(r)["repoName"]
+	repoUrl := r.FormValue("url")
+	checkDefault := r.FormValue("default")
+	opts := metav1.UpdateOptions{}
+	var repo v1alpha1.PackageRepository
+	var defaultRepo *v1alpha1.PackageRepository
+	var err error
+
+	if err := s.pkgClient.PackageRepositories().Get(r.Context(), repoName, &repo); err != nil {
+		s.sendToast(w, toast.WithErr(fmt.Errorf("failed to fetch repositories: %w", err)))
+		return
+	}
+
+	if repoUrl != "" {
+		if _, err := url.ParseRequestURI(repoUrl); err != nil {
+			s.sendToast(w, toast.WithErr(fmt.Errorf("use a valid URL for the package repository (got %v)", err)))
+			return
+		}
+		repo.Spec.Url = repoUrl
+	}
+
+	repo.Spec.Auth = nil
+
+	if checkDefault == "on" {
+		defaultRepo, err = cliutils.GetDefaultRepo(r.Context())
+		if errors.Is(err, cliutils.NoDefaultRepo) {
+			repo.SetDefaultRepository()
+		} else if err != nil {
+			s.sendToast(w, toast.WithErr(fmt.Errorf("failed to fetch repositories: %w", err)))
+			return
+		} else if defaultRepo.Name != repoName {
+			defaultRepo.SetDefaultRepositoryBool(false)
+			if err := s.pkgClient.PackageRepositories().Update(r.Context(), defaultRepo, opts); err != nil {
+				s.sendToast(w, toast.WithErr(fmt.Errorf(" error updating current default package repository: %v", err)))
+				return
+			}
+			repo.SetDefaultRepository()
+		}
+	}
+
+	if err := s.pkgClient.PackageRepositories().Update(r.Context(), &repo, opts); err != nil {
+		s.sendToast(w, toast.WithErr(fmt.Errorf(" error updating the package repository: %v", err)))
+		if checkDefault == "on" && defaultRepo != nil && defaultRepo.Name != repoName {
+			defaultRepo.SetDefaultRepositoryBool(true)
+			if err := s.pkgClient.PackageRepositories().Update(r.Context(), defaultRepo, opts); err != nil {
+				s.sendToast(w, toast.WithErr(fmt.Errorf(" error rolling back to default package repository: %v", err)))
+			}
+		}
+		return
+	}
+	s.swappingRedirect(w, "/settings", "main", "main")
+}
+
 func (s *server) enrichPage(r *http.Request, data map[string]any, err error) map[string]any {
 	data["CloudId"] = telemetry.GetMachineId()
 	if pathParts := strings.Split(r.URL.Path, "/"); len(pathParts) >= 2 {
@@ -1341,8 +1451,8 @@ func defaultKubeconfigExists() bool {
 
 func (s *server) initClusterPackageStoreAndController(ctx context.Context) (cache.Store, cache.Controller) {
 	pkgClient := s.nonCachedClient
-	return cache.NewInformer(
-		&cache.ListWatch{
+	return cache.NewInformerWithOptions(cache.InformerOptions{
+		ListerWatcher: &cache.ListWatch{
 			ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 				var pkgList v1alpha1.ClusterPackageList
 				err := pkgClient.ClusterPackages().GetAll(ctx, &pkgList)
@@ -1352,9 +1462,8 @@ func (s *server) initClusterPackageStoreAndController(ctx context.Context) (cach
 				return pkgClient.ClusterPackages().Watch(ctx, options)
 			},
 		},
-		&v1alpha1.ClusterPackage{},
-		0,
-		cache.ResourceEventHandlerFuncs{
+		ObjectType: &v1alpha1.ClusterPackage{},
+		Handler: cache.ResourceEventHandlerFuncs{
 			AddFunc: func(obj any) {
 				if pkg, ok := obj.(*v1alpha1.ClusterPackage); ok {
 					s.broadcaster.UpdatesAvailableForPackage(nil, pkg)
@@ -1378,13 +1487,13 @@ func (s *server) initClusterPackageStoreAndController(ctx context.Context) (cach
 				}
 			},
 		},
-	)
+	})
 }
 
 func (s *server) initPackageStoreAndController(ctx context.Context) (cache.Store, cache.Controller) {
 	pkgClient := s.nonCachedClient
-	return cache.NewInformer(
-		&cache.ListWatch{
+	return cache.NewInformerWithOptions(cache.InformerOptions{
+		ListerWatcher: &cache.ListWatch{
 			ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 				var pkgList v1alpha1.PackageList
 				err := pkgClient.Packages("").GetAll(ctx, &pkgList)
@@ -1394,9 +1503,8 @@ func (s *server) initPackageStoreAndController(ctx context.Context) (cache.Store
 				return pkgClient.Packages("").Watch(ctx, options)
 			},
 		},
-		&v1alpha1.Package{},
-		0,
-		cache.ResourceEventHandlerFuncs{
+		ObjectType: &v1alpha1.Package{},
+		Handler: cache.ResourceEventHandlerFuncs{
 			AddFunc: func(obj any) {
 				if pkg, ok := obj.(*v1alpha1.Package); ok {
 					s.broadcaster.UpdatesAvailableForPackage(nil, pkg)
@@ -1420,13 +1528,13 @@ func (s *server) initPackageStoreAndController(ctx context.Context) (cache.Store
 				}
 			},
 		},
-	)
+	})
 }
 
 func (s *server) initPackageInfoStoreAndController(ctx context.Context) (cache.Store, cache.Controller) {
 	pkgClient := s.nonCachedClient
-	return cache.NewInformer(
-		&cache.ListWatch{
+	return cache.NewInformerWithOptions(cache.InformerOptions{
+		ListerWatcher: &cache.ListWatch{
 			ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 				var packageInfoList v1alpha1.PackageInfoList
 				err := pkgClient.PackageInfos().GetAll(ctx, &packageInfoList)
@@ -1436,16 +1544,15 @@ func (s *server) initPackageInfoStoreAndController(ctx context.Context) (cache.S
 				return pkgClient.PackageInfos().Watch(ctx, options)
 			},
 		},
-		&v1alpha1.PackageInfo{},
-		0,
-		cache.ResourceEventHandlerFuncs{},
-	)
+		ObjectType: &v1alpha1.PackageInfo{},
+		Handler:    cache.ResourceEventHandlerFuncs{},
+	})
 }
 
 func (s *server) initPackageRepoStoreAndController(ctx context.Context) (cache.Store, cache.Controller) {
 	pkgClient := s.nonCachedClient
-	return cache.NewInformer(
-		&cache.ListWatch{
+	return cache.NewInformerWithOptions(cache.InformerOptions{
+		ListerWatcher: &cache.ListWatch{
 			ListFunc: func(options metav1.ListOptions) (runtime.Object, error) {
 				var repositoryList v1alpha1.PackageRepositoryList
 				err := pkgClient.PackageRepositories().GetAll(ctx, &repositoryList)
@@ -1455,10 +1562,9 @@ func (s *server) initPackageRepoStoreAndController(ctx context.Context) (cache.S
 				return pkgClient.PackageRepositories().Watch(ctx, options)
 			},
 		},
-		&v1alpha1.PackageRepository{},
-		0,
-		cache.ResourceEventHandlerFuncs{}, // TODO we might also want to update here?
-	)
+		ObjectType: &v1alpha1.PackageRepository{},
+		Handler:    cache.ResourceEventHandlerFuncs{}, // TODO we might also want to update here?
+	})
 }
 
 func (s *server) isUpdateAvailableForPkg(ctx context.Context, pkg ctrlpkg.Package) bool {
